#!/usr/bin/env python
#coding=utf-8

import onelab, sys
import math

OL = onelab.client(__file__)

# Display geometry and check parameters in the .geo
OL.openProject(OL.checkPath('patch.geo'))

if OL.action=='check':
    OL.run('mesher', 'gmsh - ' +  OL.checkPath('patch.geo'))

P = OL.getNumber('1Parameters/2Patch/1Radius') * 1e-3; #m
HP = OL.getNumber('1Parameters/2Patch/2Thickness') * 1e-3; #m

#=== Define additional Onelab Parameters =============================
OL.defineNumber('1Parameters/1Simulation/3FarBoundary', value=1,
                label='Far boundary Condition',
                labels = {1:'Neumann', 2:'Dirichlet'}, visible=0)
OL.defineNumber('1Parameters/1Simulation/4BottomBoundary', value=2,
                label='Bottom boundary Condition',
                labels = {1:'Neumann', 2:'Dirichlet'}, visible=0)

OL.defineNumber('1Parameters/1Simulation/1TimeStep', value=0.5,
                label='Time Step [h]')
OL.defineNumber('1Parameters/1Simulation/2FinalTime', value=1,
                label='Simulation Time [days]')

OL.defineNumber('1Parameters/3Skin/4SinkFactor',value=1e7, label='Sink Factor',
                attributes={'Highlight':'AliceBlue'})

Restart = OL.defineNumber('1Parameters/1Simulation/0Restart',value=0,
                          choices = [0, 1])


Drug = OL.defineNumber('1Parameters/4Drug/1Drug', value=4, label='Name',
        labels={1:'Nicotine',
                2:'Norelgestromin',
                3:'Fentanyl',
                4:'user defined'})

OL.defineNumber('1Parameters/4Drug/4PatchDiffusivity', value=3.46e-13, 
                label='Diffusivity in Patch [m2/s]',
                attributes={'Highlight':'LightGreen'})
OL.defineNumber('1Parameters/4Drug/2EpidermisDiffusivity', value=2.68e-12,
                 label='Diffusivity in Epidermis [m2/s]',
                 attributes={'Highlight':'LightGreen'})
OL.defineNumber('1Parameters/4Drug/3DermisDiffusivity', value=2.68e-12, 
                label='Diffusivity in Dermis [m2/s]',
                attributes={'Highlight':'LightGreen'})
OL.defineNumber('1Parameters/2Patch/5InitialConcentration', value=1.66,
                label='Initial Surf. Conc. [mg/cm2]',
                 attributes={'Highlight':'LightGreen'})

if Drug == 1 :
    OL.setNumber('1Parameters/4Drug/4PatchDiffusivity', value=3.46e-13, readOnly=1)
    OL.setNumber('1Parameters/4Drug/2EpidermisDiffusivity', value=2.68e-12, readOnly=1)
    OL.setNumber('1Parameters/4Drug/3DermisDiffusivity', value=2.68e-12, readOnly=1)
    OL.setNumber('1Parameters/2Patch/5InitialConcentration', value=1.66, readOnly=1)
elif Drug == 2 :
    OL.setNumber('1Parameters/4Drug/4PatchDiffusivity', value=1.11e-14, readOnly=1)
    OL.setNumber('1Parameters/4Drug/2EpidermisDiffusivity', value=1.11e-11, readOnly=1)
    OL.setNumber('1Parameters/4Drug/3DermisDiffusivity', value=1.11e-11, readOnly=1)
    OL.setNumber('1Parameters/2Patch/5InitialConcentration', value=0.72e-3, readOnly=1)
elif Drug == 3 :
    OL.setNumber('1Parameters/4Drug/4PatchDiffusivity', value=1e-13, readOnly=1)
    OL.setNumber('1Parameters/4Drug/2EpidermisDiffusivity', value=2e-11, readOnly=1)
    OL.setNumber('1Parameters/4Drug/3DermisDiffusivity', value=2e-11, readOnly=1)
    OL.setNumber('1Parameters/2Patch/5InitialConcentration', value=0.4, readOnly=1)
elif Drug == 4 :
    OL.setNumber('1Parameters/4Drug/4PatchDiffusivity', readOnly=0)
    OL.setNumber('1Parameters/4Drug/2EpidermisDiffusivity', readOnly=0)
    OL.setNumber('1Parameters/4Drug/3DermisDiffusivity', readOnly=0)
    OL.setNumber('1Parameters/2Patch/5InitialConcentration', readOnly=0)

InitUser = OL.getNumber('1Parameters/2Patch/5InitialConcentration')

PatchSurf = math.pi*P*P*1e4 #cm2
PatchVol = math.pi*P*P*HP #m3
InitQuantity = PatchSurf*InitUser #mg
InitConc = InitQuantity*1e-6/PatchVol #mg/cm3 = kg/m3

OL.setNumber('1Parameters/2Patch/3Surface',
             value=PatchSurf, readOnly=1,
             label='Surface [cm2]',
             attributes={'Highlight':'AliceBlue'})
OL.setNumber('1Parameters/2Patch/4Volume', label='Volume [cm3]',
             value=PatchVol*1e6, readOnly=1,
             attributes={'Highlight':'AliceBlue'})
OL.setNumber('1Parameters/2Patch/6InitialQuantity',
             value=InitQuantity, readOnly=1,
             label='Initial Drug Quantity [mg]', 
             attributes={'Highlight':'AliceBlue'})
OL.setNumber('1Parameters/2Patch/7InitialVolConcentration',
             value=InitConc, readOnly=1,
             label='Initial Vol. Conc. [kg/m3=mg/cm3]',
             attributes={'Highlight':'AliceBlue'})

# Preprocess .ol files
OL.preProcess('elmer', OL.checkPath('patch.sif.ol'))

# If we're done with the "check" phase
if OL.action == 'check' :
  	exit(1)

if not Restart :
    # Mesh the geometry and displa dit the mesh in the GUI
    OL.run('mesher', 'gmsh - ' +  OL.checkPath('patch.geo'),'-3')
    OL.mergeFile(OL.checkPath('patch.msh'))

    # Call Elmer clients, Elmer and display the field solution in the GUI
    OL.call('grid', 'ElmerGrid 14 2 patch.msh -out mesh', rundir=OL.wdir)

OL.call('elmer', 'ElmerSolver patch.sif', rundir=OL.wdir)

OL.mergeFile(OL.checkPath('solution.pos'))

# Pre-post-processing
OL.call('post', 'gmsh solution.pos script_pre.opt -', rundir=OL.wdir)

InitialQuantity = onelab.extract(OL.checkPath('initialquantity.txt'), -1, 8)
MaxDermis = onelab.extract(OL.checkPath('maximum.txt'), -1, 8)

OL.defineNumber('3Postprocessing/6InitialQuantity', value=InitialQuantity,
                 readOnly=1, attributes={'Highlight':'Pink'},
                 label='Initial drug quantity')
OL.defineNumber('3Postprocessing/7MaximumDermis', value=MaxDermis,
                readOnly=1, attributes={'Highlight':'Pink'},
                label='Max. Conc. in Dermis [mg/cm3]')

# Call postprocessing client
OL.call('post2', 'gmsh solution.pos script_transfer.opt -', rundir=OL.wdir)

import os

if not Restart :
    os.system('rm -f ndermis.txt nepidermis.txt npatch.txt ntotal.txt')

os.system('cat dermis.txt >> ndermis.txt')
os.system('cat epidermis.txt >> nepidermis.txt')
os.system('cat total.txt >> ntotal.txt')
os.system('cat patch.txt >> npatch.txt')

# Call gnuplot client
OL.call('plot', 'gnuplot plot_transfer.plt', rundir=OL.wdir)



if OL.batch:
    sys.stderr.write('%s: InitialQuantity = %12.5e (%12.5e)\n' %
                     (__file__, InitialQuantity, 7.74641))
    sys.stderr.write('%s: MaxDermis = %12.5e (%12.5e)\n' %
                     (__file__, MaxDermis, 11.1286))
    


