! ELMER input file
Check Keywords Warn

! Location of mesh files
Header
  Mesh DB "." "mesh"
End

Simulation
  Max Output Level = 3
  Coordinate System = Axi Symmetric 
  Simulation Type = Transient
  Timestepping Method = BDF
  BDF Order = 2
  Timestep Sizes = OL.eval(OL.get(1Parameters/1Simulation/1TimeStep)*3600)
  Timestep Intervals = OL.eval(OL.get(1Parameters/1Simulation/2FinalTime)*24 / OL.get(1Parameters/1Simulation/1TimeStep))
  Output intervals = 1
  Output File = File sol.pos
OL.if( OL.get(1Parameters/1Simulation/0Restart) == 1 )
  Restart Position = 0
  Restart File = File sol.pos
  Restart Before Initial Conditions = Logical True
OL.endif
End

!*******************************
!********* Equations ***********
!*******************************

Equation 1
 Active Solvers(2) = 1 2
 Convection = None;
 Concentration Units = Absolute mass; !kg/m³
End

Solver 1
  Equation = Advection Diffusion
  Variable = String Drug !Nom de la variable
  Procedure = "AdvectionDiffusion" "AdvectionDiffusionSolver"
  Linear System Solver = Iterative
  Linear System Iterative Method = BiCGStab
  Linear System Max Iterations = 200
  Linear System Convergence Tolerance = 1.0e-08
  Linear System Preconditioning = ILU1
  Linear System ILUT Tolerance = 1.0e-03
!le systeme est lineaire car Dm est indepenant de Ci
  Nonlinear System Max Iterations = 1
  Nonlinear System Convergence Tolerance = 1.0e-4
  Nonlinear System Newton After Tolerance = 1.0e-3
  Nonlinear System Newton After Iterations = 10
  Nonlinear System Relaxation Factor = 1
  Steady State Convergence Tolerance = 1.0e-4
End

! Pour le post-processing
Solver 2
   !Exec solver = "After TimeStep"
   Exec Solver = after saving
   Equation = String "ResultOutput"
   Procedure = File "ResultOutputSolve" "ResultOutputSolver"
   Output File Name = File "solution.pos"
   Output Format = String "Gmsh"
   Gmsh New Format = Logical true	
   Scalar Field 1 =  String Drug	
End

!*******************************
!************ Body *************
!*******************************
Body 1 !Epidermis
  Equation = 1
  Material = 1
OL.if( OL.get(1Parameters/1Simulation/0Restart) == 0 )
  Initial Condition = 2
OL.endif
End

Body 2 !Patch
  Equation = 1
  Material = 2
  Initial Condition = 1
End

Body 3 !Dermis
  Equation = 1
  Material = 3
  Body Force = 1
OL.if( OL.get(1Parameters/1Simulation/0Restart) == 0 )
  Initial Condition = 2
OL.endif
End

!*******************************
!********* Sink terms **********
!*******************************
Body Forces 1
  Drug Diffusion Source = Variable Drug
    Real MATC "-OL.get(1Parameters/3Skin/4SinkFactor) * OL.get(1Parameters/4Drug/3DermisDiffusivity) * tx(0)"
End

!*******************************
!***** Material Properties *****
!*******************************

! Epidermis
Material 1
  Density = Real 1.0
  Drug Diffusivity = Real OL.get(1Parameters/4Drug/2EpidermisDiffusivity)
End

! Patch
Material 2
  Density = Real 1.0
  Drug Diffusivity = Real OL.get(1Parameters/4Drug/4PatchDiffusivity)
End

! Dermis
Material 3
  Density = Real 1.0
  Drug Diffusivity = Real OL.get(1Parameters/4Drug/3DermisDiffusivity)
End

!*******************************
!***** Initial Conditions ******
!*******************************

Initial Condition 1
  Drug = Real OL.get(1Parameters/2Patch/7InitialVolConcentration)
End

Initial Condition 2
  Drug = Real 0.0
End

!*******************************
!**** Boundary Conditions ******
!*******************************

! Frontiere inferieure
Boundary Condition 1
  Target Boundaries = 1
    OL.if( OL.get(1Parameters/1Simulation/4BottomBoundary) == 1) !Neumann
      Drug Flux = Real 0.0
    OL.else !Dirichlet
      Drug = Real 0.0
    OL.endif
End

! Frontiere Distante
Boundary Condition 2
  Target Boundaries = 2
    OL.if( OL.get(1Parameters/1Simulation/3FarBoundary) == 1) !Neumann
      Drug Flux = Real 0.0
    OL.else !Dirichlet
      Drug = Real 0.0
    OL.endif
End

! Frontieres a flux nul
Boundary Condition 3
  Target Boundaries = 3
  Drug Flux = Real 0.0 !condition naturelle ( = Neumann)
End

